using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace BmpToAVI
{

    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public struct AVI_RECT
    {
        public Int32 left;
        public Int32 top;
        public Int32 right;
        public Int32 bottom;
    };
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public unsafe struct AVISTREAMINFO
    {

        public Int32 fccType;
        public Int32 fccHandler;
        public Int32 dwFlags;
        public Int32 dwCaps;
        public Int16 wPriority;
        public Int16 wLanguage;
        public Int32 dwScale;
        public Int32 dwRate;
        public Int32 dwStart;
        public Int32 dwLength;
        public Int32 dwInitialFrames;
        public Int32 dwSuggestedBufferSize;
        public Int32 dwQuality;
        public Int32 dwSampleSize;
        public AVI_RECT rcFrame;
        public Int32 dwEditCount;
        public Int32 dwFormatChangeCount;
        public fixed char szName[64];

    };

    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public struct AVICOMPRESSOPTIONS
    {
        public Int32 fccType;
        public Int32 fccHandler;
        public Int32 dwKeyFrameEvery;
        public Int32 dwQuality;
        public Int32 dwBytesPerSecond;
        public Int32 dwFlags;
        public Int32 lpFormat;
        public Int32 cbFormat;
        public Int32 lpParms;
        public Int32 cbParms;
        public Int32 dwInterleaveEvery;
    } ; 

    class BmpToAVI
    {
    private IntPtr m_hWnd;
    private IntPtr pFile = IntPtr.Zero;
    private IntPtr pStream = IntPtr.Zero;
    private IntPtr psComp = IntPtr.Zero;

        private int FrameCount = 0;
        
        private const int OF_WRITE=0x00000001;
        private const int OF_CREATE=0x00001000;


        private const int ICMF_CHOOSE_KEYFRAME = 0x00000001; 
        private const int ICMF_CHOOSE_DATARATE = 0x00000002;

        private const int AVIIF_KEYFRAME = 0x00000010;

        [DllImport("avifil32.dll")]
        extern static void AVIFileInit();

        [DllImport("avifil32.dll")]
        extern static void AVIFileExit();

        [DllImport("avifil32.dll")]
        extern static int AVIFileOpen(
            ref IntPtr pfile,
            string File,
            int Mode,
            int clsidHandler);

        [DllImport("winmm.dll", EntryPoint = "mmioStringToFOURCCA")]
        extern static int mmioStringToFOURCC(
            string sz, 
            int Flags);

        [DllImport("avifil32.dll")]
        extern static int AVIFileCreateStream(
            IntPtr pfile,
            ref IntPtr pavi,
            ref AVISTREAMINFO lParam);


        [DllImport("avifil32.dll")]
        extern static int AVISaveOptions(
            IntPtr hWnd,
            int uiFlags,
            int noStreams,
            IntPtr ppavi,
            ref IntPtr ppOptions);


        [DllImport("avifil32.dll")]
        extern static int AVIMakeCompressedStream(
            ref IntPtr ppsComp, 
            IntPtr ppavi, 
            ref AVICOMPRESSOPTIONS opts, 
            int pclisidHandler);

        [DllImport("avifil32.dll")]
        extern static int AVIStreamSetFormat(
            IntPtr ppavi, 
            int lpos, 
            IntPtr lpFormat,
            int cbFormat);

        [DllImport("avifil32.dll")]
        extern static int AVIStreamWrite(
            IntPtr ppavi,
            int lStart, 
            int lSamples, 
            IntPtr buffer, 
            int cbBuffer,
            int dwFlags, 
            int sampwritten, 
            int bytes);


        [DllImport("avifil32.dll")]
        extern static int AVIStreamRelease(IntPtr ppavi);

        [DllImport("avifil32.dll")]
        extern static int AVIFileRelease(IntPtr pfile);

        public BmpToAVI(IntPtr hwnd)
        {
            m_hWnd = hwnd;
            AVIFileInit();
        }

        public void FirstFrame(string AVIfile, string BMPfile)
        {
            
            int result = AVIFileOpen(ref pFile,
                AVIfile,
                OF_WRITE | OF_CREATE,
                0);
            RawBitmap bm = new RawBitmap();
            bm.LoadFromFile(BMPfile);

            AVISTREAMINFO Sinfo = new AVISTREAMINFO();
            Sinfo.fccType = mmioStringToFOURCC("vids", 0);
            Sinfo.fccHandler = 0;
            Sinfo.dwScale = 1;
            Sinfo.dwRate = 10;
            Sinfo.dwSuggestedBufferSize = bm.bmIH.biSizeImage;
            Sinfo.rcFrame.top = 0;
            Sinfo.rcFrame.left = 0;
            Sinfo.rcFrame.right = bm.bmIH.biWidth;
            Sinfo.rcFrame.bottom = bm.bmIH.biHeight;


            result = AVIFileCreateStream(
    pFile,
    ref pStream,
    ref Sinfo);

            IntPtr buf = Marshal.AllocHGlobal(
            Marshal.SizeOf(typeof(AVICOMPRESSOPTIONS)));

            GCHandle handle1 = GCHandle.Alloc(
        pStream,
        GCHandleType.Pinned);




            result = AVISaveOptions(
        m_hWnd,
        ICMF_CHOOSE_KEYFRAME | ICMF_CHOOSE_DATARATE,
        1,
        handle1.AddrOfPinnedObject(),
        ref buf);

            AVICOMPRESSOPTIONS opts = (AVICOMPRESSOPTIONS)
      Marshal.PtrToStructure(buf,
      typeof(AVICOMPRESSOPTIONS));
            Marshal.FreeHGlobal(buf);
            handle1.Free();


            
           
          
            result = AVIMakeCompressedStream(ref psComp, pStream, ref opts, 0);


            GCHandle handle = GCHandle.Alloc(bm.bmIH, GCHandleType.Pinned);
            result = AVIStreamSetFormat(psComp, 0, handle.AddrOfPinnedObject(), bm.bmIH.biSize);
            handle.Free();


            handle = GCHandle.Alloc(bm.bits, GCHandleType.Pinned);
            result = AVIStreamWrite(psComp,
                1,
                1,
                handle.AddrOfPinnedObject(), 
                bm.bits.GetLength(0), 
                AVIIF_KEYFRAME, 0, 0);
            FrameCount = 1;
            handle.Free();
        }

        public void NextFrame(string BMPFile)
        {
            RawBitmap bm = new RawBitmap();
            bm.LoadFromFile(BMPFile);
            GCHandle handle = GCHandle.Alloc(bm.bits, GCHandleType.Pinned);
            int result = AVIStreamWrite(psComp,
                ++FrameCount,
                1,
                handle.AddrOfPinnedObject(),
                bm.bits.GetLength(0),
                AVIIF_KEYFRAME, 0, 0);
                      }

        public void closeAll()
        {
            int result = AVIStreamRelease(psComp);
            result = AVIStreamRelease(pStream);
            result = AVIFileRelease(pFile);
        }

         ~BmpToAVI()
        {
            AVIFileExit();
        }

       
    }
}
